/*
**  main for showpic
**  show the picture of the person if there's any 
**
**  Development History:
**      who                  when           why
**      ma_muquit@fccc.edu   Nov-09-1998    first cut
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <signal.h>

#include <lber.h>
#include <ldap.h>


static void usage(char *progname);
static char *fuzzyFilter(char *s);
static void chopNL(char *string);


/*
** default stuff
*/

/* image viewer to use */
#define IMAGE_VIEWER     "xv"

/* your LDAP server */
#define MLDAP_SERVER     "ldap.fccc.edu"

/* LDAP server port */
#define MLDAP_PORT       389

/* Base DN for search */
#define MLDAP_BASEDN     "o=Fox Chase Cancer Center,c=US"


#define VERSION_S       "1.1"
#define OPTS            "vVP:S:"

int main (int argc,char **argv) 
{
    extern char
        *optarg;

    extern int
        optind;

    char
        **cnA,
        *a,
        *dn,
        **rdns,
        *attrs[3],
        **jpegphotoA,
        *opts,
        *progname,
        *filter,
        szbuf[BUFSIZ],
        *ldap_server=MLDAP_SERVER,
        *ldap_basedn=MLDAP_BASEDN;

    int
        pfd[2],
        status,
        pid,
        i,
        j,
        match,
        rc=0,
        c,
    	verbose=0,
        ldap_port=MLDAP_PORT;

    LDAPMessage
        *ent,
        *res;

    LDAP
        *ld=(LDAP *) NULL;

    BerElement
        *ber;

    struct berval
         **bvals;

    FILE
        *pp=(FILE *) NULL;

    progname=argv[0];

    while ((c=getopt(argc,argv,OPTS)) != EOF)
    {
        switch (c)
        {
            case 'P':
            {
                if (optarg == NULL)
                {
                    (void) fprintf(stderr,"port is missing with -P\n");
                    exit(1);
                }
                ldap_port=atoi(optarg);
                break;
            }
            case 'S':
            {
                if (optarg == NULL)
                {
                    (void) fprintf(stderr,"server is missing with -S\n");
                    exit(1);
                }
                ldap_server=strdup(optarg);
                if (ldap_server == NULL)
                {
                    (void) fprintf(stderr,"malloc failed for ldap_server\n");
                    exit(1);
                }
                break;
            }
            case 'h':
            {
                usage(argv[0]);
                /*
                ** won't be here
                */
                break;
            }
            case 'v':
            {
                (void) fprintf(stderr,"showpic_ldap version: %s\n",VERSION_S);
#ifdef NETSCAPE_LDAP
                (void) fprintf(stderr,"Compiled with Netscape LDAP SDK\n");      
#endif
                exit(0);
            }

            case 'V':
            {
                verbose=1;
                break;
            }

            default:
            {
                usage(argv[0]);
                /*
                ** won't be here
                */
            }
        }
    }

    if (argc - optind < 1)
    {
        usage(progname);
        exit(1);
    }
    opts=argv[optind];
    filter=fuzzyFilter(opts);
    if (filter == (char *) NULL)
    {
        (void) fprintf(stderr,"malloc failed for filter!\n");
        exit(1);
    }

    if (verbose)
    {
       (void) fprintf(stderr,"===========================================\n");
       (void) fprintf(stderr,"LDAP server: %s\n",ldap_server);
       (void) fprintf(stderr,"LDAP port:   %d\n",ldap_port);
       (void) fprintf(stderr,"Search root: %s\n",ldap_basedn);
       (void) fprintf(stderr,"Filter: %s\n",filter);
       (void) fprintf(stderr,"===========================================\n\n");
    }

    /*
    ** connect to LDAP server
    */
    ld=ldap_open(ldap_server,ldap_port);
    if (ld == (LDAP *) NULL)
    {
        (void) fprintf(stderr,"Failed to connect to LDAP server (%d): %s\n",
                       ldap_port,ldap_server);
        goto ExitProcessing;
    }

    /*
    ** bind
    */
    if (ldap_simple_bind_s(ld,ldap_basedn,NULL) != LDAP_SUCCESS)
    {
        ldap_perror(ld,"ldap_bind");
        exit(1);
    }

    /*
    ** search
    */
    attrs[0]="cn";
    attrs[1]="jpegphoto";
    attrs[2]=NULL;
    rc=ldap_search_s(ld,ldap_basedn,LDAP_SCOPE_SUBTREE,filter,
            attrs,
            0,
            &res);
    if (rc == -1)
    {
        ldap_perror(ld,"ldap_search_s");
        goto ExitProcessing;
    }

    /*
    ** how many matches
    */
    match=ldap_count_entries(ld,res);
    (void) fprintf(stderr,"Number of names matched: %d\n",match);


    for (ent=ldap_first_entry(ld,res); ent != 0;ent=ldap_next_entry(ld,ent))
    {
        rdns=NULL;
        dn=ldap_get_dn(ld,ent);
        if (dn)
        {
            rdns=ldap_explode_dn(dn,1);
        }
        else
            continue;

        cnA=ldap_get_values(ld,ent,"cn");
        if (cnA == NULL)
        {
             continue;
        }
        jpegphotoA=ldap_get_values(ld,ent,"jpegphoto");
        if (jpegphotoA != NULL)
        {

            (void) fprintf(stderr,"Display picture of %s? [y|n|q]: ",*rdns);
            (void) fgets(szbuf,sizeof(szbuf),stdin);
            chopNL(szbuf);
            if (*szbuf == 'y' || *szbuf == 'Y')
            {
                bvals=ldap_get_values_len(ld,ent,"jpegphoto");
                if (bvals)
                {
                   pipe(pfd);

                   pid=fork();

                   switch (pid)
                   {
                       case -1:
                       {
                           perror("fork");
                           break;
                       }

                       case 0:
                       {
                           close(fileno(stdin));
                           dup(pfd[0]);
                           close(pfd[0]);
                           close(pfd[1]);
                           execlp(IMAGE_VIEWER,IMAGE_VIEWER,"-",NULL);
                           break;
                       }
                   }
                   close(pfd[0]);
                   write(pfd[1],bvals[0]->bv_val,bvals[0]->bv_len);
                   close(pfd[1]);
                   while (wait(&status) != pid)
                       ;
                }
            }
            else if (*szbuf == 'q' || *szbuf == 'Q')
                goto ExitProcessing;
        }
        else
        {
                if (rdns)
                    (void) fprintf(stderr,"No picture found for: %s\n",*rdns);
        }
        if (rdns)
            (void) ldap_value_free(rdns);
        if (dn)
            (void) free(dn);
    }


ExitProcessing:
    if (ld)
        ldap_unbind(ld);

    return(0);
}

static void usage(char *progname)
{
    (void) fprintf(stderr, 
            "showpic_ldap - show the picture of the person if any\n");
    (void) fprintf(stderr,"by ma_muquit@fccc.edu\n");
    (void) fprintf(stderr,"\nusage: %s [options] <string>\n",progname);
    (void) fprintf(stderr,"where the options include:\n");
    (void) fprintf(stderr,"     -S server   LDAP server\n");
    (void) fprintf(stderr,"     -P port     Port on LDAP server\n");
    (void) fprintf(stderr,"     -h          Shows this help\n");
    (void) fprintf(stderr,"     -v          Print version\n");
    (void) fprintf(stderr,"     -V          Verbose\n\n");
    exit(0);
}



/*
** mallocs, caller must free it
*/
static char *fuzzyFilter(char *s)
{
    register char
        *p;

    char
        *filter=(char *) NULL;
    int
        its_phone_num=0,
        its_room_number=0,
        its_email=0,
        all_digits=0,
        digit_found=0;

    filter=malloc(BUFSIZ*sizeof(char));
    if (filter == (char *) NULL)
    {
        (void) fprintf(stderr," malloc failed\n");
        return ((char *) NULL);
    }
    /* 
    ** if *, just return it
    */
    if (strncmp(s,"*",1) == 0)
    {
        filter=strdup("(objectclass=top)");
        return(filter);
    }

    /*
    ** if all are digits, it is a phone number
    */
    for (p=s; *p != '\0'; p++)
    {
        if (!isdigit(*p))
            break;
        else
            all_digits=1;
    }
    /*
    ** if (all not digits, see if first one is ascii and the rest is digits,
    ** if so, it's a room number
    */
    if (!all_digits)
    {
        /*
        ** we'r counting few meta characters
        */
        if ((s[0] > 64) && (s[0] <= 124))
        {
            /*
            ** if the rest are digits, it is a room number
            */
            for (p=s; *p != '\0'; p++)
            {
                if (isdigit(*p))
                {
                    its_room_number=1;
                    break;
                }
            }
        }
    }

    /*
    ** if _ is found, its a email
    */
    for (p=s; *p != '\0'; p++)
    {
        if (*p == '_')
        {
            its_email=1;
            break;
        }
    }
    if (its_email)
    {
        (void) strcpy(filter,"(mail=*");
        (void) strcat(filter,s);
        (void) strcat(filter,"*)");
        return (filter);
    }

    if (its_room_number)
    {
        (void) strcpy(filter,"(roomnumber=*");
        (void) strcat(filter,s);
        (void) strcat(filter,"*)");
        return (filter);
    }

    if (all_digits)
    {
        (void) strcpy(filter,"(telephonenumber=*");
        (void) strcat(filter,s);
        (void) strcat(filter,"*)");
        return (filter);
    }

    /*
    ** just search cn
    */
    (void) sprintf(filter,"(cn=*%s*)",s);
    return (filter);
}

static void chopNL(char *string)
{
    char
        c;
    char
        *ptr;

    c='\0';

    for (ptr = string; *ptr; ptr++);

    if (ptr != string)
    {
        c = *(--ptr);

        if (c == '\n')
        {
            *ptr='\0';
        }
    }

}

